from __future__ import annotations

import logging
from dataclasses import dataclass
from pathlib import Path
from typing import TypedDict, cast

import anyio
import click
import mutagen.mp3

from .base import Speaker, SynthesizeError, common_options, filter_none
from .consts import MINIMAX_SUPPORTED_VOICES

logger = logging.getLogger(__name__)


class TimberWeight(TypedDict):
    voice_id: str
    """系统音色"""
    weight: int
    """取值[1,100]。最多支持4种音色混合，取值为整数，单一音色取值占比越高，合成音色越像。"""


@dataclass
class MinimaxSpeaker(Speaker):
    """MiniMax TTS speaker.

    Args:
        api_key (str): The MiniMax API key.
        group_id (str): The MiniMax group ID.
        model (str): The model to use. Defaults to "speech-01".
        voice (str): The voice to use.
        timber_weights (list[TimberWeight]): The timber weights.
        speed (float): The speed of speech. Range [0.5, 2.0]. Defaults to 1.0.
        vol (float | int): The volume of speech. Range (0, 10]. Defaults to 1.
        pitch (int): The pitch of speech. Range [-12, 12]. Defaults to 0.
    """

    api_key: str
    group_id: str
    model: str = "speech-01"
    voice: str | None = None
    timber_weights: list[TimberWeight] | None = None
    speed: float | None = None
    vol: float | int | None = None
    pitch: int | None = None

    async def synthesize(
        self, text: str, out_file: str | Path, lang: str = "en-US"
    ) -> float:
        import httpx

        data = filter_none(
            {
                "model": self.model,
                "voice_id": self.voice or self.list_voices()[0],
                "timber_weights": self.timber_weights,
                "speed": self.speed,
                "vol": self.vol,
                "pitch": self.pitch,
                "text": text,
            }
        )

        async with httpx.AsyncClient(
            headers={"Authorization": f"Bearer {self.api_key}"},
            params={"GroupId": self.group_id},
        ) as client:
            async with client.stream(
                "POST", "https://api.minimax.chat/v1/text_to_speech", json=data
            ) as resp:
                if resp.is_error:
                    await resp.aread()
                    logger.error("Failed to get tts: %s", resp.text)
                    raise SynthesizeError("Failed to get tts")
                if "application/json" in resp.headers["Content-Type"]:
                    logger.error("Failed to get tts: %s", await resp.aread())
                    err = resp.json()
                    raise SynthesizeError(
                        "Failed to get tts: "
                        f"{err.get('base_resp', {}).get('status_msg', '')}"
                    )
                file = anyio.Path(out_file)
                async with await file.open("wb") as f:
                    async for chunk in resp.aiter_bytes(8192):
                        await f.write(chunk)

        audio = mutagen.mp3.MP3(out_file)
        return cast(float, audio.info.length)

    @classmethod
    def list_voices(cls) -> list[str]:
        return MINIMAX_SUPPORTED_VOICES

    @classmethod
    def get_command(cls) -> click.Command:
        @click.command()
        @click.option(
            "--api-key",
            required=True,
            envvar="MINIMAX_API_KEY",
            help="The Minimax API key.",
            show_envvar=True,
        )
        @click.option(
            "--group-id",
            required=True,
            help="The Minimax group ID.",
            show_envvar=True,
            envvar="MINIMAX_GROUP_ID",
        )
        @click.option(
            "--speed", default=1.0, help="The speed of speech.", type=click.FLOAT
        )
        @click.option(
            "--vol", default=1, help="The volume of speech.", type=click.FLOAT
        )
        @click.option("--pitch", default=0, help="The pitch of speech.", type=click.INT)
        @common_options(cls)
        def minimax(
            api_key: str,
            group_id: str,
            voice: str | None,
            speed: float,
            vol: float,
            pitch: int,
            text: str,
            lang: str,
            output: str,
        ):
            speaker = MinimaxSpeaker(
                api_key, group_id, voice=voice, speed=speed, vol=vol, pitch=pitch
            )
            speaker.say(text, output, lang=lang)

        return minimax



"""Synthesizes speech from the input string of text."""

from __future__ import annotations

import json
import os
from pathlib import Path
from typing import cast

import anyio
import click
import mutagen.mp3
from google.cloud import texttospeech

from .base import Speaker, common_options
from .consts import GOOGLE_SUPPORTED_VOICES


class GoogleSpeaker(Speaker):
    """Google TTS speaker.

    Args:
        voice (str): The voice to use. Defaults to "en-US-Studio-M".
        speaking_rate (float):
            Optional. Input only. Speaking rate/speed, in the range
            [0.25, 4.0]. 1.0 is the normal native speed supported by the
            specific voice. 2.0 is twice as fast, and 0.5 is half as
            fast. If unset(0.0), defaults to the native 1.0 speed. Any
            other values < 0.25 or > 4.0 will return an error.
        pitch (float):
            Optional. Input only. Speaking pitch, in the range [-20.0,
            20.0]. 20 means increase 20 semitones from the original
            pitch. -20 means decrease 20 semitones from the original
            pitch.
        volume_gain_db (float):
            Optional. Input only. Volume gain (in dB) of the normal
            native volume supported by the specific voice, in the range
            [-96.0, 16.0]. If unset, or set to a value of 0.0 (dB), will
            play at normal native signal amplitude. A value of -6.0 (dB)
            will play at approximately half the amplitude of the normal
            native signal amplitude. A value of +6.0 (dB) will play at
            approximately twice the amplitude of the normal native
            signal amplitude. Strongly recommend not to exceed +10 (dB)
            as there's usually no effective increase in loudness for any
            value greater than that.
    """

    def __init__(
        self,
        *,
        voice: str | None = None,
        speaking_rate: float = 1.0,
        pitch: float = 0.0,
        volume_gain_db: float = 0.0,
    ) -> None:
        self.voice = voice
        self.audio_config = texttospeech.AudioConfig(
            audio_encoding=texttospeech.AudioEncoding.MP3,
            speaking_rate=speaking_rate,
            pitch=pitch,
            volume_gain_db=volume_gain_db,
        )

    def get_voice(self, lang: str) -> str:
        if self.voice:
            return self.voice
        else:
            if lang.startswith("zh-"):
                lang = "cmn-" + lang[3:]
            return next(
                (v for v in self.list_voices() if v.startswith(lang)),
                "en-US-Studio-M",
            )

    async def synthesize(
        self, text: str, out_file: str | Path, lang: str = "en-US"
    ) -> float:
        input_text = texttospeech.SynthesisInput(text=text)
        voice = texttospeech.VoiceSelectionParams(
            name=(voice := self.get_voice(lang)),
            language_code="-".join(voice.split("-")[:2]),
        )
        if "GOOGLE_CREDENTIALS_JSON" in os.environ:
            from google.oauth2 import service_account

            credentials = service_account.Credentials.from_service_account_info(
                json.loads(os.environ["GOOGLE_CREDENTIALS_JSON"])
            )
        else:
            credentials = None  # Use default credentials
        async with texttospeech.TextToSpeechAsyncClient(
            credentials=credentials
        ) as client:
            resp = await client.synthesize_speech(
                request={
                    "input": input_text,
                    "voice": voice,
                    "audio_config": self.audio_config,
                }
            )
        file = anyio.Path(out_file)
        async with await file.open("wb") as f:
            await f.write(resp.audio_content)

        audio = mutagen.mp3.MP3(out_file)
        return cast(float, audio.info.length)

    @classmethod
    def list_voices(cls) -> list[str]:
        return GOOGLE_SUPPORTED_VOICES

    @classmethod
    def get_command(cls) -> click.Command:
        @click.command()
        @click.option(
            "--speaking-rate", default=1.0, help="The speaking rate.", type=click.FLOAT
        )
        @click.option("--pitch", default=0.0, help="The pitch.", type=click.FLOAT)
        @click.option(
            "--volume-gain-db", default=0.0, help="The volume gain.", type=click.FLOAT
        )
        @common_options(cls)
        def google(
            voice: str | None,
            speaking_rate: float,
            pitch: float,
            volume_gain_db: float,
            text: str,
            lang: str,
            output: str,
        ) -> None:
            speaker = cls(
                voice=voice,
                speaking_rate=speaking_rate,
                pitch=pitch,
                volume_gain_db=volume_gain_db,
            )
            speaker.say(text, output, lang=lang)

        return google



MICROSOFT_SUPPORTED_VOICES = [
    "af-ZA-AdriNeural",
    "af-ZA-WillemNeural",
    "am-ET-AmehaNeural",
    "am-ET-MekdesNeural",
    "ar-AE-FatimaNeural",
    "ar-AE-HamdanNeural",
    "ar-BH-AliNeural",
    "ar-BH-LailaNeural",
    "ar-DZ-AminaNeural",
    "ar-DZ-IsmaelNeural",
    "ar-EG-SalmaNeural",
    "ar-EG-ShakirNeural",
    "ar-IQ-BasselNeural",
    "ar-IQ-RanaNeural",
    "ar-JO-SanaNeural",
    "ar-JO-TaimNeural",
    "ar-KW-FahedNeural",
    "ar-KW-NouraNeural",
    "ar-LB-LaylaNeural",
    "ar-LB-RamiNeural",
    "ar-LY-ImanNeural",
    "ar-LY-OmarNeural",
    "ar-MA-JamalNeural",
    "ar-MA-MounaNeural",
    "ar-OM-AbdullahNeural",
    "ar-OM-AyshaNeural",
    "ar-QA-AmalNeural",
    "ar-QA-MoazNeural",
    "ar-SA-HamedNeural",
    "ar-SA-ZariyahNeural",
    "ar-SY-AmanyNeural",
    "ar-SY-LaithNeural",
    "ar-TN-HediNeural",
    "ar-TN-ReemNeural",
    "ar-YE-MaryamNeural",
    "ar-YE-SalehNeural",
    "az-AZ-BabekNeural",
    "az-AZ-BanuNeural",
    "bg-BG-BorislavNeural",
    "bg-BG-KalinaNeural",
    "bn-BD-NabanitaNeural",
    "bn-BD-PradeepNeural",
    "bn-IN-BashkarNeural",
    "bn-IN-TanishaaNeural",
    "bs-BA-GoranNeural",
    "bs-BA-VesnaNeural",
    "ca-ES-EnricNeural",
    "ca-ES-JoanaNeural",
    "cs-CZ-AntoninNeural",
    "cs-CZ-VlastaNeural",
    "cy-GB-AledNeural",
    "cy-GB-NiaNeural",
    "da-DK-ChristelNeural",
    "da-DK-JeppeNeural",
    "de-AT-IngridNeural",
    "de-AT-JonasNeural",
    "de-CH-JanNeural",
    "de-CH-LeniNeural",
    "de-DE-AmalaNeural",
    "de-DE-ConradNeural",
    "de-DE-FlorianMultilingualNeural",
    "de-DE-KatjaNeural",
    "de-DE-KillianNeural",
    "de-DE-SeraphinaMultilingualNeural",
    "el-GR-AthinaNeural",
    "el-GR-NestorasNeural",
    "en-AU-NatashaNeural",
    "en-AU-WilliamNeural",
    "en-CA-ClaraNeural",
    "en-CA-LiamNeural",
    "en-GB-LibbyNeural",
    "en-GB-MaisieNeural",
    "en-GB-RyanNeural",
    "en-GB-SoniaNeural",
    "en-GB-ThomasNeural",
    "en-HK-SamNeural",
    "en-HK-YanNeural",
    "en-IE-ConnorNeural",
    "en-IE-EmilyNeural",
    "en-IN-NeerjaExpressiveNeural",
    "en-IN-NeerjaNeural",
    "en-IN-PrabhatNeural",
    "en-KE-AsiliaNeural",
    "en-KE-ChilembaNeural",
    "en-NG-AbeoNeural",
    "en-NG-EzinneNeural",
    "en-NZ-MitchellNeural",
    "en-NZ-MollyNeural",
    "en-PH-JamesNeural",
    "en-PH-RosaNeural",
    "en-SG-LunaNeural",
    "en-SG-WayneNeural",
    "en-TZ-ElimuNeural",
    "en-TZ-ImaniNeural",
    "en-US-AnaNeural",
    "en-US-AndrewMultilingualNeural",
    "en-US-AndrewNeural",
    "en-US-AriaNeural",
    "en-US-AvaMultilingualNeural",
    "en-US-AvaNeural",
    "en-US-BrianMultilingualNeural",
    "en-US-BrianNeural",
    "en-US-ChristopherNeural",
    "en-US-EmmaMultilingualNeural",
    "en-US-EmmaNeural",
    "en-US-EricNeural",
    "en-US-GuyNeural",
    "en-US-JennyNeural",
    "en-US-MichelleNeural",
    "en-US-RogerNeural",
    "en-US-SteffanNeural",
    "en-ZA-LeahNeural",
    "en-ZA-LukeNeural",
    "es-AR-ElenaNeural",
    "es-AR-TomasNeural",
    "es-BO-MarceloNeural",
    "es-BO-SofiaNeural",
    "es-CL-CatalinaNeural",
    "es-CL-LorenzoNeural",
    "es-CO-GonzaloNeural",
    "es-CO-SalomeNeural",
    "es-CR-JuanNeural",
    "es-CR-MariaNeural",
    "es-CU-BelkysNeural",
    "es-CU-ManuelNeural",
    "es-DO-EmilioNeural",
    "es-DO-RamonaNeural",
    "es-EC-AndreaNeural",
    "es-EC-LuisNeural",
    "es-ES-AlvaroNeural",
    "es-ES-ElviraNeural",
    "es-ES-XimenaNeural",
    "es-GQ-JavierNeural",
    "es-GQ-TeresaNeural",
    "es-GT-AndresNeural",
    "es-GT-MartaNeural",
    "es-HN-CarlosNeural",
    "es-HN-KarlaNeural",
    "es-MX-DaliaNeural",
    "es-MX-JorgeNeural",
    "es-NI-FedericoNeural",
    "es-NI-YolandaNeural",
    "es-PA-MargaritaNeural",
    "es-PA-RobertoNeural",
    "es-PE-AlexNeural",
    "es-PE-CamilaNeural",
    "es-PR-KarinaNeural",
    "es-PR-VictorNeural",
    "es-PY-MarioNeural",
    "es-PY-TaniaNeural",
    "es-SV-LorenaNeural",
    "es-SV-RodrigoNeural",
    "es-US-AlonsoNeural",
    "es-US-PalomaNeural",
    "es-UY-MateoNeural",
    "es-UY-ValentinaNeural",
    "es-VE-PaolaNeural",
    "es-VE-SebastianNeural",
    "et-EE-AnuNeural",
    "et-EE-KertNeural",
    "fa-IR-DilaraNeural",
    "fa-IR-FaridNeural",
    "fi-FI-HarriNeural",
    "fi-FI-NooraNeural",
    "fil-PH-AngeloNeural",
    "fil-PH-BlessicaNeural",
    "fr-BE-CharlineNeural",
    "fr-BE-GerardNeural",
    "fr-CA-AntoineNeural",
    "fr-CA-JeanNeural",
    "fr-CA-SylvieNeural",
    "fr-CA-ThierryNeural",
    "fr-CH-ArianeNeural",
    "fr-CH-FabriceNeural",
    "fr-FR-DeniseNeural",
    "fr-FR-EloiseNeural",
    "fr-FR-HenriNeural",
    "fr-FR-RemyMultilingualNeural",
    "fr-FR-VivienneMultilingualNeural",
    "ga-IE-ColmNeural",
    "ga-IE-OrlaNeural",
    "gl-ES-RoiNeural",
    "gl-ES-SabelaNeural",
    "gu-IN-DhwaniNeural",
    "gu-IN-NiranjanNeural",
    "he-IL-AvriNeural",
    "he-IL-HilaNeural",
    "hi-IN-MadhurNeural",
    "hi-IN-SwaraNeural",
    "hr-HR-GabrijelaNeural",
    "hr-HR-SreckoNeural",
    "hu-HU-NoemiNeural",
    "hu-HU-TamasNeural",
    "id-ID-ArdiNeural",
    "id-ID-GadisNeural",
    "is-IS-GudrunNeural",
    "is-IS-GunnarNeural",
    "it-IT-DiegoNeural",
    "it-IT-ElsaNeural",
    "it-IT-GiuseppeNeural",
    "it-IT-IsabellaNeural",
    "ja-JP-KeitaNeural",
    "ja-JP-NanamiNeural",
    "jv-ID-DimasNeural",
    "jv-ID-SitiNeural",
    "ka-GE-EkaNeural",
    "ka-GE-GiorgiNeural",
    "kk-KZ-AigulNeural",
    "kk-KZ-DauletNeural",
    "km-KH-PisethNeural",
    "km-KH-SreymomNeural",
    "kn-IN-GaganNeural",
    "kn-IN-SapnaNeural",
    "ko-KR-HyunsuNeural",
    "ko-KR-InJoonNeural",
    "ko-KR-SunHiNeural",
    "lo-LA-ChanthavongNeural",
    "lo-LA-KeomanyNeural",
    "lt-LT-LeonasNeural",
    "lt-LT-OnaNeural",
    "lv-LV-EveritaNeural",
    "lv-LV-NilsNeural",
    "mk-MK-AleksandarNeural",
    "mk-MK-MarijaNeural",
    "ml-IN-MidhunNeural",
    "ml-IN-SobhanaNeural",
    "mn-MN-BataaNeural",
    "mn-MN-YesuiNeural",
    "mr-IN-AarohiNeural",
    "mr-IN-ManoharNeural",
    "ms-MY-OsmanNeural",
    "ms-MY-YasminNeural",
    "mt-MT-GraceNeural",
    "mt-MT-JosephNeural",
    "my-MM-NilarNeural",
    "my-MM-ThihaNeural",
    "nb-NO-FinnNeural",
    "nb-NO-PernilleNeural",
    "ne-NP-HemkalaNeural",
    "ne-NP-SagarNeural",
    "nl-BE-ArnaudNeural",
    "nl-BE-DenaNeural",
    "nl-NL-ColetteNeural",
    "nl-NL-FennaNeural",
    "nl-NL-MaartenNeural",
    "pl-PL-MarekNeural",
    "pl-PL-ZofiaNeural",
    "ps-AF-GulNawazNeural",
    "ps-AF-LatifaNeural",
    "pt-BR-AntonioNeural",
    "pt-BR-FranciscaNeural",
    "pt-BR-ThalitaNeural",
    "pt-PT-DuarteNeural",
    "pt-PT-RaquelNeural",
    "ro-RO-AlinaNeural",
    "ro-RO-EmilNeural",
    "ru-RU-DmitryNeural",
    "ru-RU-SvetlanaNeural",
    "si-LK-SameeraNeural",
    "si-LK-ThiliniNeural",
    "sk-SK-LukasNeural",
    "sk-SK-ViktoriaNeural",
    "sl-SI-PetraNeural",
    "sl-SI-RokNeural",
    "so-SO-MuuseNeural",
    "so-SO-UbaxNeural",
    "sq-AL-AnilaNeural",
    "sq-AL-IlirNeural",
    "sr-RS-NicholasNeural",
    "sr-RS-SophieNeural",
    "su-ID-JajangNeural",
    "su-ID-TutiNeural",
    "sv-SE-MattiasNeural",
    "sv-SE-SofieNeural",
    "sw-KE-RafikiNeural",
    "sw-KE-ZuriNeural",
    "sw-TZ-DaudiNeural",
    "sw-TZ-RehemaNeural",
    "ta-IN-PallaviNeural",
    "ta-IN-ValluvarNeural",
    "ta-LK-KumarNeural",
    "ta-LK-SaranyaNeural",
    "ta-MY-KaniNeural",
    "ta-MY-SuryaNeural",
    "ta-SG-AnbuNeural",
    "ta-SG-VenbaNeural",
    "te-IN-MohanNeural",
    "te-IN-ShrutiNeural",
    "th-TH-NiwatNeural",
    "th-TH-PremwadeeNeural",
    "tr-TR-AhmetNeural",
    "tr-TR-EmelNeural",
    "uk-UA-OstapNeural",
    "uk-UA-PolinaNeural",
    "ur-IN-GulNeural",
    "ur-IN-SalmanNeural",
    "ur-PK-AsadNeural",
    "ur-PK-UzmaNeural",
    "uz-UZ-MadinaNeural",
    "uz-UZ-SardorNeural",
    "vi-VN-HoaiMyNeural",
    "vi-VN-NamMinhNeural",
    "zh-CN-XiaoxiaoNeural",
    "zh-CN-XiaoyiNeural",
    "zh-CN-YunjianNeural",
    "zh-CN-YunxiNeural",
    "zh-CN-YunxiaNeural",
    "zh-CN-YunyangNeural",
    "zh-CN-liaoning-XiaobeiNeural",
    "zh-CN-shaanxi-XiaoniNeural",
    "zh-HK-HiuGaaiNeural",
    "zh-HK-HiuMaanNeural",
    "zh-HK-WanLungNeural",
    "zh-TW-HsiaoChenNeural",
    "zh-TW-HsiaoYuNeural",
    "zh-TW-YunJheNeural",
    "zu-ZA-ThandoNeural",
    "zu-ZA-ThembaNeural",
]
EDGE_SUPPORTED_VOICES = [*MICROSOFT_SUPPORTED_VOICES, "en-IN-NeerjaExpressiveNeural"]
AZURE_SUPPORTED_VOICES = [
    *MICROSOFT_SUPPORTED_VOICES,
    "ca-ES-AlbaNeural",
    "de-DE-BerndNeural",
    "de-DE-ChristophNeural",
    "de-DE-ElkeNeural",
    "de-DE-GiselaNeural",
    "de-DE-KasperNeural",
    "de-DE-KlarissaNeural",
    "de-DE-KlausNeural",
    "de-DE-LouisaNeural",
    "de-DE-MajaNeural",
    "de-DE-RalfNeural",
    "de-DE-TanjaNeural",
    "en-AU-AnnetteNeural",
    "en-AU-CarlyNeural",
    "en-AU-DarrenNeural",
    "en-AU-DuncanNeural",
    "en-AU-ElsieNeural",
    "en-AU-FreyaNeural",
    "en-AU-JoanneNeural",
    "en-AU-KenNeural",
    "en-AU-KimNeural",
    "en-AU-NeilNeural",
    "en-AU-TimNeural",
    "en-AU-TinaNeural",
    "en-GB-AbbiNeural",
    "en-GB-AlfieNeural",
    "en-GB-BellaNeural",
    "en-GB-ElliotNeural",
    "en-GB-EthanNeural",
    "en-GB-HollieNeural",
    "en-GB-NoahNeural",
    "en-GB-OliverNeural",
    "en-GB-OliviaNeural",
    "en-GB-MiaNeural",
    "en-IN-AaravNeural",
    "en-IN-AashiNeural",
    "en-IN-AnanyaNeural",
    "en-IN-KavyaNeural",
    "en-IN-KunalNeural",
    "en-IN-RehaanNeural",
    "en-US-DavisNeural",
    "en-US-JaneNeural",
    "en-US-JasonNeural",
    "en-US-SaraNeural",
    "en-US-TonyNeural",
    "en-US-NancyNeural",
    "en-US-AmberNeural",
    "en-US-AshleyNeural",
    "en-US-BrandonNeural",
    "en-US-CoraNeural",
    "en-US-ElizabethNeural",
    "en-US-JacobNeural",
    "en-US-JennyMultilingualNeural",
    "en-US-MonicaNeural",
    "en-US-RyanMultilingualNeural",
    "en-US-AIGenerate1Neural",
    "en-US-AIGenerate2Neural",
    "en-US-BlueNeural",
    "es-ES-AbrilNeural",
    "es-ES-ArnauNeural",
    "es-ES-DarioNeural",
    "es-ES-EliasNeural",
    "es-ES-EstrellaNeural",
    "es-ES-IreneNeural",
    "es-ES-LaiaNeural",
    "es-ES-LiaNeural",
    "es-ES-NilNeural",
    "es-ES-SaulNeural",
    "es-ES-TeoNeural",
    "es-ES-TrianaNeural",
    "es-ES-VeraNeural",
    "es-MX-BeatrizNeural",
    "es-MX-CandelaNeural",
    "es-MX-CarlotaNeural",
    "es-MX-CecilioNeural",
    "es-MX-GerardoNeural",
    "es-MX-LarissaNeural",
    "es-MX-LibertoNeural",
    "es-MX-LucianoNeural",
    "es-MX-MarinaNeural",
    "es-MX-NuriaNeural",
    "es-MX-PelayoNeural",
    "es-MX-RenataNeural",
    "es-MX-YagoNeural",
    "eu-ES-AinhoaNeural",
    "eu-ES-AnderNeural",
    "fi-FI-SelmaNeural",
    "fr-FR-AlainNeural",
    "fr-FR-BrigitteNeural",
    "fr-FR-CelesteNeural",
    "fr-FR-ClaudeNeural",
    "fr-FR-CoralieNeural",
    "fr-FR-JacquelineNeural",
    "fr-FR-JeromeNeural",
    "fr-FR-JosephineNeural",
    "fr-FR-MauriceNeural",
    "fr-FR-YvesNeural",
    "fr-FR-YvetteNeural",
    "hi-IN-AaravNeural",
    "hi-IN-AnanyaNeural",
    "hi-IN-KavyaNeural",
    "hi-IN-KunalNeural",
    "hi-IN-RehaanNeural",
    "hy-AM-AnahitNeural",
    "hy-AM-HaykNeural",
    "it-IT-BenignoNeural",
    "it-IT-CalimeroNeural",
    "it-IT-CataldoNeural",
    "it-IT-FabiolaNeural",
    "it-IT-FiammaNeural",
    "it-IT-GianniNeural",
    "it-IT-ImeldaNeural",
    "it-IT-IrmaNeural",
    "it-IT-LisandroNeural",
    "it-IT-PalmiraNeural",
    "it-IT-PierinaNeural",
    "it-IT-RinaldoNeural",
    "ja-JP-AoiNeural",
    "ja-JP-DaichiNeural",
    "ja-JP-MayuNeural",
    "ja-JP-NaokiNeural",
    "ja-JP-ShioriNeural",
    "ja-JP-MasaruMultilingualNeural",
    "ko-KR-BongJinNeural",
    "ko-KR-GookMinNeural",
    "ko-KR-JiMinNeural",
    "ko-KR-SeoHyeonNeural",
    "ko-KR-SoonBokNeural",
    "ko-KR-YuJinNeural",
    "nb-NO-IselinNeural",
    "pl-PL-AgnieszkaNeural",
    "pt-BR-BrendaNeural",
    "pt-BR-DonatoNeural",
    "pt-BR-ElzaNeural",
    "pt-BR-FabioNeural",
    "pt-BR-GiovannaNeural",
    "pt-BR-HumbertoNeural",
    "pt-BR-JulioNeural",
    "pt-BR-LeilaNeural",
    "pt-BR-LeticiaNeural",
    "pt-BR-ManuelaNeural",
    "pt-BR-NicolauNeural",
    "pt-BR-ValerioNeural",
    "pt-BR-YaraNeural",
    "pt-PT-FernandaNeural",
    "ru-RU-DariyaNeural",
    "sr-Latn-RS-NicholasNeural",
    "sr-Latn-RS-SophieNeural",
    "sv-SE-HilleviNeural",
    "th-TH-AcharaNeural",
    "wuu-CN-XiaotongNeural",
    "wuu-CN-YunzheNeural",
    "yue-CN-XiaoMinNeural",
    "yue-CN-YunSongNeural",
    "zh-CN-XiaochenNeural",
    "zh-CN-XiaohanNeural",
    "zh-CN-XiaomengNeural",
    "zh-CN-XiaomoNeural",
    "zh-CN-XiaoqiuNeural",
    "zh-CN-XiaoruiNeural",
    "zh-CN-XiaoshuangNeural",
    "zh-CN-XiaoxiaoMultilingualNeural",
    "zh-CN-XiaoyanNeural",
    "zh-CN-XiaoyouNeural",
    "zh-CN-XiaozhenNeural",
    "zh-CN-YunfengNeural",
    "zh-CN-YunhaoNeural",
    "zh-CN-YunyeNeural",
    "zh-CN-YunzeNeural",
    "zh-CN-XiaochenMultilingualNeural",
    "zh-CN-XiaorouNeural",
    "zh-CN-XiaoxiaoDialectsNeural",
    "zh-CN-XiaoyuMultilingualNeural",
    "zh-CN-YunjieNeural",
    "zh-CN-YunyiMultilingualNeural",
    "zh-CN-XiaoxuanNeural",
    "zh-CN-guangxi-YunqiNeural",
    "zh-CN-henan-YundengNeural",
    "zh-CN-liaoning-YunbiaoNeural",
    "zh-CN-shandong-YunxiangNeural",
    "zh-CN-sichuan-YunxiNeural",
]

VOLC_SUPPORTED_VOICES = [
    "zh_female_qingxin",
    "zh_male_chunhou",
    "zh_female_zhixing",
    "zh_male_qinqie",
    "zh_female_qiaopi",
    "zh_female_tianmei",
    "zh_male_novel_narration",
    "zh_male_changtianyi_xuanyi",
    "zh_female_sistermango",
    "zh_female_xiaoyue",
    "zh_female_changshengpopo",
    "zh_female_wuzetian",
    "zh_male_narration",
    "zh_female_xiaoqian",
    "zh_male_xiaojian",
    "zh_female_story",
    "zh_male_xiaohai",
    "zh_male_huoli",
    "zh_female_xiaopengyou",
    "zh_male_sunwukong_clone2",
    "zh_male_rap",
    "zh_male_rap_bgm_00",
    "zh_male_rap_bgm_01",
    "zh_female_sichuan",
    "zh_female_mengyatou",
    "zh_male_yuanqizhengtai",
    "zh_male_silang",
    "zh_male_xionger",
    "zh_female_guli",
    "zh_female_xiaowanzi",
    "zh_female_peiqi",
    "zh_male_liangjiahui",
    "zh_female_xiaoke",
    "zh_female_dblizhi",
    "zh_female_dbqiangshi",
    "zh_female_zhubo",
    "zh_male_zhubo",
    "zh_male_ad",
    "zh_male_commentate",
    "zh_male_inspirational",
    "zh_female_inspirational",
    "zh_male_xiaoming",
    "zh_male_dongyue_clone2",
    "zh_male_chunhui",
    "en_male_adam",
    "en_female_sarah",
    "en_male_dryw",
    "en_female_emotional",
    "en_male_narration",
    "en_male_funny",
    "en_male_mason",
    "en_male_ryan",
    "en_male_smith",
    "en_male_cody",
    "en_female_samc",
    "en_male_grinch",
    "en_male_wizard",
    "en_female_grandma",
    "en_female_richgirl",
    "en_male_cupid",
    "jp_female_hana",
    "jp_male_satoshi",
]
GOOGLE_SUPPORTED_VOICES = [
    "af-ZA-Standard-A",
    "am-ET-Standard-A",
    "am-ET-Standard-B",
    "am-ET-Wavenet-A",
    "am-ET-Wavenet-B",
    "ar-XA-Standard-A",
    "ar-XA-Standard-B",
    "ar-XA-Standard-C",
    "ar-XA-Standard-D",
    "ar-XA-Wavenet-A",
    "ar-XA-Wavenet-B",
    "ar-XA-Wavenet-C",
    "ar-XA-Wavenet-D",
    "bg-BG-Standard-A",
    "bn-IN-Standard-A",
    "bn-IN-Standard-B",
    "bn-IN-Standard-C",
    "bn-IN-Standard-D",
    "bn-IN-Wavenet-A",
    "bn-IN-Wavenet-B",
    "bn-IN-Wavenet-C",
    "bn-IN-Wavenet-D",
    "ca-ES-Standard-A",
    "cmn-CN-Standard-A",
    "cmn-CN-Standard-B",
    "cmn-CN-Standard-C",
    "cmn-CN-Standard-D",
    "cmn-CN-Wavenet-A",
    "cmn-CN-Wavenet-B",
    "cmn-CN-Wavenet-C",
    "cmn-CN-Wavenet-D",
    "cmn-TW-Standard-A",
    "cmn-TW-Standard-B",
    "cmn-TW-Standard-C",
    "cmn-TW-Wavenet-A",
    "cmn-TW-Wavenet-B",
    "cmn-TW-Wavenet-C",
    "cs-CZ-Standard-A",
    "cs-CZ-Wavenet-A",
    "da-DK-Neural2-D",
    "da-DK-Standard-A",
    "da-DK-Standard-C",
    "da-DK-Standard-D",
    "da-DK-Standard-E",
    "da-DK-Wavenet-A",
    "da-DK-Wavenet-C",
    "da-DK-Wavenet-D",
    "da-DK-Wavenet-E",
    "de-DE-Neural2-A",
    "de-DE-Neural2-B",
    "de-DE-Neural2-C",
    "de-DE-Neural2-D",
    "de-DE-Neural2-F",
    "de-DE-Polyglot-1",
    "de-DE-Standard-A",
    "de-DE-Standard-B",
    "de-DE-Standard-C",
    "de-DE-Standard-D",
    "de-DE-Standard-E",
    "de-DE-Standard-F",
    "de-DE-Studio-B",
    "de-DE-Studio-C",
    "de-DE-Wavenet-A",
    "de-DE-Wavenet-B",
    "de-DE-Wavenet-C",
    "de-DE-Wavenet-D",
    "de-DE-Wavenet-E",
    "de-DE-Wavenet-F",
    "el-GR-Standard-A",
    "el-GR-Wavenet-A",
    "en-AU-Neural2-A",
    "en-AU-Neural2-B",
    "en-AU-Neural2-C",
    "en-AU-Neural2-D",
    "en-AU-News-E",
    "en-AU-News-F",
    "en-AU-News-G",
    "en-AU-Polyglot-1",
    "en-AU-Standard-A",
    "en-AU-Standard-B",
    "en-AU-Standard-C",
    "en-AU-Standard-D",
    "en-AU-Wavenet-A",
    "en-AU-Wavenet-B",
    "en-AU-Wavenet-C",
    "en-AU-Wavenet-D",
    "en-GB-Neural2-A",
    "en-GB-Neural2-B",
    "en-GB-Neural2-C",
    "en-GB-Neural2-D",
    "en-GB-Neural2-F",
    "en-GB-News-G",
    "en-GB-News-H",
    "en-GB-News-I",
    "en-GB-News-J",
    "en-GB-News-K",
    "en-GB-News-L",
    "en-GB-News-M",
    "en-GB-Standard-A",
    "en-GB-Standard-B",
    "en-GB-Standard-C",
    "en-GB-Standard-D",
    "en-GB-Standard-F",
    "en-GB-Studio-B",
    "en-GB-Studio-C",
    "en-GB-Wavenet-A",
    "en-GB-Wavenet-B",
    "en-GB-Wavenet-C",
    "en-GB-Wavenet-D",
    "en-GB-Wavenet-F",
    "en-IN-Neural2-A",
    "en-IN-Neural2-B",
    "en-IN-Neural2-C",
    "en-IN-Neural2-D",
    "en-IN-Standard-A",
    "en-IN-Standard-B",
    "en-IN-Standard-C",
    "en-IN-Standard-D",
    "en-IN-Wavenet-A",
    "en-IN-Wavenet-B",
    "en-IN-Wavenet-C",
    "en-IN-Wavenet-D",
    "en-US-Casual-K",
    "en-US-Journey-D",
    "en-US-Journey-F",
    "en-US-Neural2-A",
    "en-US-Neural2-C",
    "en-US-Neural2-D",
    "en-US-Neural2-E",
    "en-US-Neural2-F",
    "en-US-Neural2-G",
    "en-US-Neural2-H",
    "en-US-Neural2-I",
    "en-US-Neural2-J",
    "en-US-News-K",
    "en-US-News-L",
    "en-US-News-N",
    "en-US-Polyglot-1",
    "en-US-Standard-A",
    "en-US-Standard-B",
    "en-US-Standard-C",
    "en-US-Standard-D",
    "en-US-Standard-E",
    "en-US-Standard-F",
    "en-US-Standard-G",
    "en-US-Standard-H",
    "en-US-Standard-I",
    "en-US-Standard-J",
    "en-US-Studio-O",
    "en-US-Studio-Q",
    "en-US-Wavenet-A",
    "en-US-Wavenet-B",
    "en-US-Wavenet-C",
    "en-US-Wavenet-D",
    "en-US-Wavenet-E",
    "en-US-Wavenet-F",
    "en-US-Wavenet-G",
    "en-US-Wavenet-H",
    "en-US-Wavenet-I",
    "en-US-Wavenet-J",
    "es-ES-Neural2-A",
    "es-ES-Neural2-B",
    "es-ES-Neural2-C",
    "es-ES-Neural2-D",
    "es-ES-Neural2-E",
    "es-ES-Neural2-F",
    "es-ES-Polyglot-1",
    "es-ES-Standard-A",
    "es-ES-Standard-B",
    "es-ES-Standard-C",
    "es-ES-Standard-D",
    "es-ES-Wavenet-B",
    "es-ES-Wavenet-C",
    "es-ES-Wavenet-D",
    "es-US-Neural2-A",
    "es-US-Neural2-B",
    "es-US-Neural2-C",
    "es-US-News-D",
    "es-US-News-E",
    "es-US-News-F",
    "es-US-News-G",
    "es-US-Polyglot-1",
    "es-US-Standard-A",
    "es-US-Standard-B",
    "es-US-Standard-C",
    "es-US-Studio-B",
    "es-US-Wavenet-A",
    "es-US-Wavenet-B",
    "es-US-Wavenet-C",
    "eu-ES-Standard-A",
    "fi-FI-Standard-A",
    "fi-FI-Wavenet-A",
    "fil-PH-Standard-A",
    "fil-PH-Standard-B",
    "fil-PH-Standard-C",
    "fil-PH-Standard-D",
    "fil-PH-Wavenet-A",
    "fil-PH-Wavenet-B",
    "fil-PH-Wavenet-C",
    "fil-PH-Wavenet-D",
    "fil-ph-Neural2-A",
    "fil-ph-Neural2-D",
    "fr-CA-Neural2-A",
    "fr-CA-Neural2-B",
    "fr-CA-Neural2-C",
    "fr-CA-Neural2-D",
    "fr-CA-Standard-A",
    "fr-CA-Standard-B",
    "fr-CA-Standard-C",
    "fr-CA-Standard-D",
    "fr-CA-Wavenet-A",
    "fr-CA-Wavenet-B",
    "fr-CA-Wavenet-C",
    "fr-CA-Wavenet-D",
    "fr-FR-Neural2-A",
    "fr-FR-Neural2-B",
    "fr-FR-Neural2-C",
    "fr-FR-Neural2-D",
    "fr-FR-Neural2-E",
    "fr-FR-Polyglot-1",
    "fr-FR-Standard-A",
    "fr-FR-Standard-B",
    "fr-FR-Standard-C",
    "fr-FR-Standard-D",
    "fr-FR-Standard-E",
    "fr-FR-Studio-A",
    "fr-FR-Studio-D",
    "fr-FR-Wavenet-A",
    "fr-FR-Wavenet-B",
    "fr-FR-Wavenet-C",
    "fr-FR-Wavenet-D",
    "fr-FR-Wavenet-E",
    "gl-ES-Standard-A",
    "gu-IN-Standard-A",
    "gu-IN-Standard-B",
    "gu-IN-Standard-C",
    "gu-IN-Standard-D",
    "gu-IN-Wavenet-A",
    "gu-IN-Wavenet-B",
    "gu-IN-Wavenet-C",
    "gu-IN-Wavenet-D",
    "he-IL-Standard-A",
    "he-IL-Standard-B",
    "he-IL-Standard-C",
    "he-IL-Standard-D",
    "he-IL-Wavenet-A",
    "he-IL-Wavenet-B",
    "he-IL-Wavenet-C",
    "he-IL-Wavenet-D",
    "hi-IN-Neural2-A",
    "hi-IN-Neural2-B",
    "hi-IN-Neural2-C",
    "hi-IN-Neural2-D",
    "hi-IN-Standard-A",
    "hi-IN-Standard-B",
    "hi-IN-Standard-C",
    "hi-IN-Standard-D",
    "hi-IN-Wavenet-A",
    "hi-IN-Wavenet-B",
    "hi-IN-Wavenet-C",
    "hi-IN-Wavenet-D",
    "hu-HU-Standard-A",
    "hu-HU-Wavenet-A",
    "id-ID-Standard-A",
    "id-ID-Standard-B",
    "id-ID-Standard-C",
    "id-ID-Standard-D",
    "id-ID-Wavenet-A",
    "id-ID-Wavenet-B",
    "id-ID-Wavenet-C",
    "id-ID-Wavenet-D",
    "is-IS-Standard-A",
    "it-IT-Neural2-A",
    "it-IT-Neural2-C",
    "it-IT-Standard-A",
    "it-IT-Standard-B",
    "it-IT-Standard-C",
    "it-IT-Standard-D",
    "it-IT-Wavenet-A",
    "it-IT-Wavenet-B",
    "it-IT-Wavenet-C",
    "it-IT-Wavenet-D",
    "ja-JP-Neural2-B",
    "ja-JP-Neural2-C",
    "ja-JP-Neural2-D",
    "ja-JP-Standard-A",
    "ja-JP-Standard-B",
    "ja-JP-Standard-C",
    "ja-JP-Standard-D",
    "ja-JP-Wavenet-A",
    "ja-JP-Wavenet-B",
    "ja-JP-Wavenet-C",
    "ja-JP-Wavenet-D",
    "kn-IN-Standard-A",
    "kn-IN-Standard-B",
    "kn-IN-Standard-C",
    "kn-IN-Standard-D",
    "kn-IN-Wavenet-A",
    "kn-IN-Wavenet-B",
    "kn-IN-Wavenet-C",
    "kn-IN-Wavenet-D",
    "ko-KR-Neural2-A",
    "ko-KR-Neural2-B",
    "ko-KR-Neural2-C",
    "ko-KR-Standard-A",
    "ko-KR-Standard-B",
    "ko-KR-Standard-C",
    "ko-KR-Standard-D",
    "ko-KR-Wavenet-A",
    "ko-KR-Wavenet-B",
    "ko-KR-Wavenet-C",
    "ko-KR-Wavenet-D",
    "lt-LT-Standard-A",
    "lv-LV-Standard-A",
    "ml-IN-Standard-A",
    "ml-IN-Standard-B",
    "ml-IN-Standard-C",
    "ml-IN-Standard-D",
    "ml-IN-Wavenet-A",
    "ml-IN-Wavenet-B",
    "ml-IN-Wavenet-C",
    "ml-IN-Wavenet-D",
    "mr-IN-Standard-A",
    "mr-IN-Standard-B",
    "mr-IN-Standard-C",
    "mr-IN-Wavenet-A",
    "mr-IN-Wavenet-B",
    "mr-IN-Wavenet-C",
    "ms-MY-Standard-A",
    "ms-MY-Standard-B",
    "ms-MY-Standard-C",
    "ms-MY-Standard-D",
    "ms-MY-Wavenet-A",
    "ms-MY-Wavenet-B",
    "ms-MY-Wavenet-C",
    "ms-MY-Wavenet-D",
    "nb-NO-Standard-A",
    "nb-NO-Standard-B",
    "nb-NO-Standard-C",
    "nb-NO-Standard-D",
    "nb-NO-Standard-E",
    "nb-NO-Wavenet-A",
    "nb-NO-Wavenet-B",
    "nb-NO-Wavenet-C",
    "nb-NO-Wavenet-D",
    "nb-NO-Wavenet-E",
    "nl-BE-Standard-A",
    "nl-BE-Standard-B",
    "nl-BE-Wavenet-A",
    "nl-BE-Wavenet-B",
    "nl-NL-Standard-A",
    "nl-NL-Standard-B",
    "nl-NL-Standard-C",
    "nl-NL-Standard-D",
    "nl-NL-Standard-E",
    "nl-NL-Wavenet-A",
    "nl-NL-Wavenet-B",
    "nl-NL-Wavenet-C",
    "nl-NL-Wavenet-D",
    "nl-NL-Wavenet-E",
    "pa-IN-Standard-A",
    "pa-IN-Standard-B",
    "pa-IN-Standard-C",
    "pa-IN-Standard-D",
    "pa-IN-Wavenet-A",
    "pa-IN-Wavenet-B",
    "pa-IN-Wavenet-C",
    "pa-IN-Wavenet-D",
    "pl-PL-Standard-A",
    "pl-PL-Standard-B",
    "pl-PL-Standard-C",
    "pl-PL-Standard-D",
    "pl-PL-Standard-E",
    "pl-PL-Wavenet-A",
    "pl-PL-Wavenet-B",
    "pl-PL-Wavenet-C",
    "pl-PL-Wavenet-D",
    "pl-PL-Wavenet-E",
    "pt-BR-Neural2-A",
    "pt-BR-Neural2-B",
    "pt-BR-Neural2-C",
    "pt-BR-Standard-A",
    "pt-BR-Standard-B",
    "pt-BR-Standard-C",
    "pt-BR-Wavenet-A",
    "pt-BR-Wavenet-B",
    "pt-BR-Wavenet-C",
    "pt-PT-Standard-A",
    "pt-PT-Standard-B",
    "pt-PT-Standard-C",
    "pt-PT-Standard-D",
    "pt-PT-Wavenet-A",
    "pt-PT-Wavenet-B",
    "pt-PT-Wavenet-C",
    "pt-PT-Wavenet-D",
    "ro-RO-Standard-A",
    "ro-RO-Wavenet-A",
    "ru-RU-Standard-A",
    "ru-RU-Standard-B",
    "ru-RU-Standard-C",
    "ru-RU-Standard-D",
    "ru-RU-Standard-E",
    "ru-RU-Wavenet-A",
    "ru-RU-Wavenet-B",
    "ru-RU-Wavenet-C",
    "ru-RU-Wavenet-D",
    "ru-RU-Wavenet-E",
    "sk-SK-Standard-A",
    "sk-SK-Wavenet-A",
    "sr-RS-Standard-A",
    "sv-SE-Standard-A",
    "sv-SE-Standard-B",
    "sv-SE-Standard-C",
    "sv-SE-Standard-D",
    "sv-SE-Standard-E",
    "sv-SE-Wavenet-A",
    "sv-SE-Wavenet-B",
    "sv-SE-Wavenet-C",
    "sv-SE-Wavenet-D",
    "sv-SE-Wavenet-E",
    "ta-IN-Standard-A",
    "ta-IN-Standard-B",
    "ta-IN-Standard-C",
    "ta-IN-Standard-D",
    "ta-IN-Wavenet-A",
    "ta-IN-Wavenet-B",
    "ta-IN-Wavenet-C",
    "ta-IN-Wavenet-D",
    "te-IN-Standard-A",
    "te-IN-Standard-B",
    "th-TH-Neural2-C",
    "th-TH-Standard-A",
    "tr-TR-Standard-A",
    "tr-TR-Standard-B",
    "tr-TR-Standard-C",
    "tr-TR-Standard-D",
    "tr-TR-Standard-E",
    "tr-TR-Wavenet-A",
    "tr-TR-Wavenet-B",
    "tr-TR-Wavenet-C",
    "tr-TR-Wavenet-D",
    "tr-TR-Wavenet-E",
    "uk-UA-Standard-A",
    "uk-UA-Wavenet-A",
    "vi-VN-Neural2-A",
    "vi-VN-Neural2-D",
    "vi-VN-Standard-A",
    "vi-VN-Standard-B",
    "vi-VN-Standard-C",
    "vi-VN-Standard-D",
    "vi-VN-Wavenet-A",
    "vi-VN-Wavenet-B",
    "vi-VN-Wavenet-C",
    "vi-VN-Wavenet-D",
    "yue-HK-Standard-A",
    "yue-HK-Standard-B",
    "yue-HK-Standard-C",
    "yue-HK-Standard-D",
]

BAIDU_SUPPORTED_VOICES = {
    # Basic
    "DuXiaoYu": 1,
    "DuXiaoMei": 0,
    "DuXiaoYao": 3,
    "DuYaYa": 4,
    # High Quality
    "DuXiaoYaoHQ": 5003,
    "DuXiaoLu": 5118,
    "DuBoWen": 106,
    "DuXiaoTong": 110,
    "DuMiDuo": 103,
    "DuXiaoJiao": 5,
    "DuXiaoMeng": 111,
}

MINIMAX_SUPPORTED_VOICES = [
    "male-qn-qingse",
    "male-qn-jingying",
    "male-qn-badao",
    "male-qn-daxuesheng",
    "female-shaonv",
    "female-yujie",
    "female-chengshu",
    "female-tianmei",
    "presenter_male",
    "presenter_female",
    "audiobook_male_1",
    "audiobook_male_2",
    "audiobook_female_1",
    "audiobook_female_2",
    "male-qn-qingse-jingpin",
    "male-qn-jingying-jingpin",
    "male-qn-badao-jingpin",
    "male-qn-daxuesheng-jingpin",
    "female-shaonv-jingpin",
    "female-yujie-jingpin",
    "female-chengshu-jingpin",
    "female-tianmei-jingpin",
]



from __future__ import annotations

from dataclasses import dataclass
from pathlib import Path

import anyio
import click

from .base import Speaker, SynthesizeError, common_options
from .consts import EDGE_SUPPORTED_VOICES


@dataclass
class EdgeSpeaker(Speaker):
    """Edge TTS speaker.

    Args:
        voice (str): The voice to use.
        rate (str): The rate of speech.
        pitch (str): The pitch of speech.
        volume (str): The volume of speech.
    """

    voice: str | None = None
    rate: str = "+0%"
    pitch: str = "+0Hz"
    volume: str = "+0%"

    async def synthesize(
        self, text: str, out_file: str | Path, lang: str = "en-US"
    ) -> float:
        import edge_tts

        communicate = edge_tts.Communicate(
            text,
            voice=self.get_voice(lang),
            rate=self.rate,
            pitch=self.pitch,
            volume=self.volume,
        )
        duration = 0
        file = anyio.Path(out_file)
        async with await file.open("wb") as f:
            async for chunk in communicate.stream():
                if chunk["type"] == "audio":
                    await f.write(chunk["data"])
                elif chunk["type"] == "WordBoundary":
                    duration = (chunk["offset"] + chunk["duration"]) / 1e7
            if duration == 0:
                raise SynthesizeError("Failed to get tts from edge")
        return duration

    def get_voice(self, lang: str) -> str:
        if self.voice:
            return self.voice
        else:
            return next(
                (v for v in self.list_voices() if v.startswith(lang)),
                "en-US-AriaNeural",
            )

    @classmethod
    def list_voices(cls) -> list[str]:
        return EDGE_SUPPORTED_VOICES

    @classmethod
    def get_command(cls) -> click.Command:
        @click.command()
        @click.option("--rate", default="+0%", help="The rate of speech.")
        @click.option("--pitch", default="+0Hz", help="The pitch of speech.")
        @click.option("--volume", default="+0%", help="The volume of speech.")
        @common_options(cls)
        def edge(
            voice: str | None,
            rate: str,
            pitch: str,
            volume: str,
            text: str,
            lang: str,
            output: str,
        ) -> None:
            speaker = cls(voice=voice, rate=rate, pitch=pitch, volume=volume)
            speaker.say(text, output, lang=lang)

        return edge



import logging

import click

from .azure import AzureSpeaker
from .baidu import BaiduSpeaker
from .edge import EdgeSpeaker
from .google import GoogleSpeaker
from .minimax import MinimaxSpeaker
from .openai import OpenAISpeaker
from .volc import VolcSpeaker


def setup_logger(debug: bool = False):
    logger = logging.getLogger("tetos")
    handler = logging.StreamHandler()
    handler.setFormatter(logging.Formatter("%(asctime)s - %(levelname)s - %(message)s"))
    logger.addHandler(handler)
    logger.setLevel(logging.DEBUG if debug else logging.INFO)


@click.group(context_settings={"show_default": True})
@click.option("--verbose", "-v", is_flag=True, help="Enable verbose logging.")
def tts(verbose: bool) -> None:
    setup_logger(verbose)


for speaker in (
    OpenAISpeaker,
    EdgeSpeaker,
    AzureSpeaker,
    VolcSpeaker,
    GoogleSpeaker,
    BaiduSpeaker,
    MinimaxSpeaker,
):
    tts.add_command(speaker.get_command())


if __name__ == "__main__":
    tts()



from __future__ import annotations

import abc
from pathlib import Path
from typing import Any, Callable, TypeVar

import click

F = TypeVar("F", bound=Callable[..., Any])


class SynthesizeError(RuntimeError):
    pass


class Speaker(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    async def synthesize(
        self, text: str, out_file: str | Path, lang: str = "en-US"
    ) -> float:
        """Generate speech from text and save it to a file.

        Args:
            text (str): The text to synthesize.
            out_file (Path): The file to save the speech to.
            lang (str): The language code of the text. e.g. "en-US", "fr-FR".

        Returns:
            float: The duration of the speech in seconds.
        """
        raise NotImplementedError

    @classmethod
    @abc.abstractmethod
    def list_voices(cls) -> list[str]:
        """List the available voices for the speaker.

        Returns:
            list[str]: A list of voice names.
        """
        raise NotImplementedError

    @classmethod
    @abc.abstractmethod
    def get_command(cls) -> click.Command:
        """Return a Click command for the speaker.

        Returns:
            click.Command: The Click command.
        """
        raise NotImplementedError

    def say(
        self, text: str, out_file: str | Path | None = None, lang: str = "en-US"
    ) -> float:
        """A synchronous version of synthesize()

        Args:
            text (str): The text to synthesize.
            out_file (Path): The file to save the speech to.
            lang (str): The language code of the text. e.g. "en-US", "fr-FR".
        """
        import anyio
        import click

        if out_file is None:
            out_file = Path("tts-output.mp3")

        result = anyio.run(self.synthesize, text, out_file, lang)
        click.echo(f"Speech is generated successfully at {out_file}")
        return result


def common_options(cls: Speaker) -> Callable[[F], F]:
    def list_voices(ctx, param, value):
        if not value or ctx.resilient_parsing:
            return
        click.echo("\n".join(cls.list_voices()))
        ctx.exit()

    def decorator(func: F) -> F:
        func = click.argument("text")(func)
        func = click.option(
            "--output",
            "-o",
            type=click.Path(dir_okay=False),
            default="tts-output.mp3",
            help="The output file.",
        )(func)
        func = click.option(
            "--voice",
            help="The voice to use. See supported voices with `--list-voices`",
        )(func)
        func = click.option(
            "--lang",
            default="en-US",
            help="The language code of the text. e.g. 'en-US', 'fr-FR'.",
        )(func)
        func = click.option(
            "--list-voices",
            "-l",
            is_flag=True,
            help="List available voices.",
            is_eager=True,
            callback=list_voices,
            expose_value=False,
        )(func)
        return func

    return decorator


def filter_none(d: dict[str, Any]) -> dict[str, Any]:
    return {k: v for k, v in d.items() if v is not None}



from __future__ import annotations

import base64
import hashlib
import hmac
import json
import logging
import time
from datetime import datetime, timezone
from pathlib import Path
from typing import Any, Generator, cast

import anyio
import click
from click.core import Command as Command
from httpx import AsyncClient, Auth, Request
from httpx._models import Response

from .base import Speaker, SynthesizeError, common_options
from .consts import VOLC_SUPPORTED_VOICES

logger = logging.getLogger(__name__)


class VolcSignAuth(Auth):
    ALGORITHM = "HMAC-SHA256"

    requires_request_body = True

    def __init__(
        self, access_key: str, secret_key: str, service: str, region: str
    ) -> None:
        self.access_key = access_key
        self.secret_key = secret_key
        self.service = service
        self.region = region

    @staticmethod
    def hmac_sha256(key: bytes, content: str) -> bytes:
        return hmac.new(key, content.encode("utf-8"), hashlib.sha256).digest()

    def auth_flow(self, request: Request) -> Generator[Request, Response, None]:
        x_content_sha256 = hashlib.sha256(request.content).hexdigest()
        x_date = datetime.now(timezone.utc).strftime("%Y%m%dT%H%M%SZ")
        signed_headers = "content-type;host;x-content-sha256;x-date"
        canonical_request = "\n".join(
            [
                request.method.upper(),
                request.url.path,
                request.url.query.decode(),
                f"content-type:{request.headers['content-type']}",
                f"host:{request.headers['host']}",
                f"x-content-sha256:{x_content_sha256}",
                f"x-date:{x_date}",
                "",
                signed_headers,
                x_content_sha256,
            ]
        )

        canonical_request_hash = hashlib.sha256(canonical_request.encode()).hexdigest()
        credential_scope = f"{x_date[:8]}/{self.region}/{self.service}/request"
        string_to_sign = "\n".join(
            [
                self.ALGORITHM,
                x_date,
                credential_scope,
                canonical_request_hash,
            ]
        )
        sign_key = self.secret_key.encode()
        for scope in credential_scope.split("/"):
            sign_key = self.hmac_sha256(sign_key, scope)
        signature = self.hmac_sha256(sign_key, string_to_sign).hex()
        authorization = (
            f"{self.ALGORITHM} Credential={self.access_key}/{credential_scope}"
            f", SignedHeaders={signed_headers}, Signature={signature}"
        )
        request.headers.update(
            {
                "Authorization": authorization,
                "X-Content-Sha256": x_content_sha256,
                "X-Date": x_date,
            }
        )
        yield request


class VolcSpeaker(Speaker):
    """Volcengine TTS speaker.

    Args:
        access_key (str): The access key ID.
        secret_key (str): The access secret key.
        app_key (str): The app key.
        voice (str, optional): The voice to use.
        sample_rate (int, optional): The sample rate.
            Available values: [8000,16000,22050,24000,32000,44100,48000],
            Defaults to 24000.
        speech_rate (int, optional): The speech rate. It should be in range [-50,100].
            100 means 2x speed and -50 means half speed. Defaults to 0.
        pitch_rate (int, optional): The pitch rate. It should be in range [-12,12].
            Defaults to 0.
    """

    SERVICE_NAME = "sami"
    REGION = "cn-north-1"
    AUTH_VERSION = "volc-auth-v1"
    API_HOST = "open.volcengineapi.com"
    VERSION = "2021-07-27"
    TOKEN_FILE = Path.home() / ".tetos" / "volc_token.json"
    SAMI_API_URL = "https://sami.bytedance.com/api/v1/invoke"

    def __init__(
        self,
        access_key: str,
        secret_key: str,
        app_key: str,
        *,
        voice: str | None = None,
        sample_rate: int = 24000,
        speech_rate: int = 0,
        pitch_rate: int = 0,
    ) -> None:
        self.access_key = access_key
        self.secret_key = secret_key
        self.app_key = app_key
        self.voice = voice or "zh_female_qingxin"
        self.sample_rate = sample_rate
        self.speech_rate = speech_rate
        self.pitch_rate = pitch_rate
        self._token: dict[str, Any] = {}
        self.client = AsyncClient()
        if self.TOKEN_FILE.exists():
            with self.TOKEN_FILE.open() as f:
                self._token = json.load(f)

    async def _ensure_token(self) -> str:
        if not self._token or time.time() > self._token["expires_at"]:
            self._token = await self._request_token()
            # self._token = self._request_token_sync()
        return self._token["token"]

    async def _request_token(self) -> dict[str, Any]:
        logger.info("Requesting token")
        data = {
            "appkey": self.app_key,
            "token_version": self.AUTH_VERSION,
            "expiration": 3600 * 24,  # expire in 1 day
        }
        resp = await self.client.post(
            f"https://{self.API_HOST}/",
            params={"Action": "GetToken", "Version": self.VERSION},
            json=data,
            headers={"Host": self.API_HOST},
            auth=VolcSignAuth(
                self.access_key, self.secret_key, self.SERVICE_NAME, self.REGION
            ),
        )
        if resp.is_error:
            logger.error("Failed to get token: %s", resp.text)
            raise SynthesizeError("Failed to get token")
        token = resp.json()
        self.TOKEN_FILE.parent.mkdir(parents=True, exist_ok=True)
        with self.TOKEN_FILE.open("w") as f:
            json.dump(token, f)
        return token

    async def synthesize(
        self, text: str, out_file: str | Path, lang: str = "en-US"
    ) -> float:
        tts_payload = json.dumps(
            {
                "text": text,
                "speaker": self.voice,
                "audio_config": {
                    "format": "mp3",
                    "sample_rate": self.sample_rate,
                    "speech_rate": self.speech_rate,
                    "pitch_rate": self.pitch_rate,
                },
            }
        )
        req = {
            "appkey": self.app_key,
            "token": await self._ensure_token(),
            "namespace": "TTS",
            "payload": tts_payload,
        }

        resp = await self.client.post(self.SAMI_API_URL, json=req)
        if resp.is_error:
            raise self._get_error(resp)
        data = resp.json()
        if data["status_code"] == 20000000 and len(data["data"]) > 0:
            file = anyio.Path(out_file)
            async with await file.open("wb") as f:
                await f.write(base64.b64decode(data["data"]))
            payload = json.loads(data["payload"])
            return cast(float, payload["duration"])
        raise SynthesizeError("Failed to get tts from volcengine")

    def _get_error(self, resp: Response) -> SynthesizeError:
        logger.error("Failed to get tts from volcengine: %s", resp.text)
        try:
            data = resp.json()
            return SynthesizeError(data["status_text"])
        except Exception:
            return SynthesizeError("Failed to get tts from volcengine")

    @classmethod
    def list_voices(cls) -> list[str]:
        return VOLC_SUPPORTED_VOICES

    @classmethod
    def get_command(cls) -> click.Command:
        @click.command()
        @click.option(
            "--access-key",
            required=True,
            help="The access key.",
            envvar="VOLC_ACCESS_KEY",
            show_envvar=True,
        )
        @click.option(
            "--secret-key",
            required=True,
            help="The secret key.",
            envvar="VOLC_SECRET_KEY",
            show_envvar=True,
        )
        @click.option(
            "--app-key",
            required=True,
            help="The app key.",
            envvar="VOLC_APP_KEY",
            show_envvar=True,
        )
        @click.option("--voice", default="zh_female_qingxin", help="The voice to use.")
        @click.option(
            "--sample-rate", default=24000, help="The sample rate.", type=click.INT
        )
        @click.option(
            "--speech-rate", default=0, help="The speech rate.", type=click.INT
        )
        @click.option("--pitch-rate", default=0, help="The pitch rate.", type=click.INT)
        @common_options(cls)
        def volc(
            access_key: str,
            secret_key: str,
            app_key: str,
            voice: str,
            sample_rate: int,
            speech_rate: int,
            pitch_rate: int,
            lang: str,
            text: str,
            output: str,
        ) -> None:
            speaker = cls(
                access_key,
                secret_key,
                app_key,
                voice=voice,
                sample_rate=sample_rate,
                speech_rate=speech_rate,
                pitch_rate=pitch_rate,
            )
            speaker.say(text, output, lang=lang)

        return volc



from __future__ import annotations

import json
import logging
import platform
import time
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any, ClassVar, cast
from urllib.parse import quote_plus

import anyio
import click
import httpx
import mutagen.mp3

from .base import Speaker, SynthesizeError, common_options
from .consts import BAIDU_SUPPORTED_VOICES

logger = logging.getLogger(__name__)

TOKEN_URL = "https://aip.baidubce.com/oauth/2.0/token"
TTS_URL = "https://tsn.baidu.com/text2audio"
SCOPE = "audio_tts_post"
CUID = f"tetos/{platform.python_implementation()}-{platform.python_version()}"


@dataclass
class BaiduSpeaker(Speaker):
    """Baidu TTS speaker.

    Args:
        api_key (str): The Baidu API key.
        secret_key (str): The Baidu secret key.
        voice (str): The voice to use.
        speed (int): The speed of speech, from 0 to 15. Defaults to 5.
        pitch (int): The pitch of speech, from 0 to 15. Defaults to 5.
        volume (int): The volume of speech,
            from 0 to 9(basic) and 0 to 15(high quality). Defaults to 5.
    """

    api_key: str
    secret_key: str
    voice: str | None = None
    speed: int = 5
    pitch: int = 5
    volume: int = 5
    _token: dict[str, Any] = field(default_factory=dict, init=False)

    TOKEN_FILE: ClassVar[Path] = Path.home() / ".tetos" / "baidu_token.json"

    def __post_init__(self) -> None:
        if self.TOKEN_FILE.exists():
            with self.TOKEN_FILE.open() as f:
                self._token = json.load(f)

    @property
    def per(self) -> int:
        return BAIDU_SUPPORTED_VOICES[self.voice] if self.voice else 1

    async def _ensure_token(self) -> str:
        if not self._token or time.time() > self._token["expires_at"]:
            self._token = await self._request_token()
        return self._token["access_token"]

    async def _request_token(self) -> dict[str, Any]:
        logger.info("Requesting token")
        data = {
            "grant_type": "client_credentials",
            "client_id": self.api_key,
            "client_secret": self.secret_key,
        }
        async with httpx.AsyncClient() as client:
            resp = await client.post(TOKEN_URL, data=data)
            if resp.is_error:
                logger.error("Failed to get token: %s", resp.text)
                raise SynthesizeError("Failed to get token")
            token = resp.json()
            if SCOPE not in token["scope"].split():
                raise SynthesizeError("scope is not correct")
            token["expires_at"] = time.time() + token["expires_in"]
            with self.TOKEN_FILE.open("w") as f:
                json.dump(token, f)
            return token

    async def synthesize(
        self, text: str, out_file: str | Path, lang: str = "en-US"
    ) -> float:
        params = {
            "tok": await self._ensure_token(),
            "tex": quote_plus(text),  # one more encode here
            "per": self.per,
            "spd": self.speed,
            "pit": self.pitch,
            "vol": self.volume,
            "aue": 3,  # mp3
            "cuid": CUID,
            "lan": "zh",  # Fixed value
            "ctp": 1,  # Fixed value
        }
        async with httpx.AsyncClient() as client:
            async with client.stream("POST", TTS_URL, data=params) as resp:
                if resp.is_error:
                    await resp.aread()
                    logger.error("Failed to get tts: %s", resp.text)
                    raise SynthesizeError("Failed to get tts")
                file = anyio.Path(out_file)
                async with await file.open("wb") as f:
                    async for chunk in resp.aiter_bytes(8192):
                        await f.write(chunk)

        audio = mutagen.mp3.MP3(out_file)
        return cast(float, audio.info.length)

    @classmethod
    def list_voices(cls) -> list[str]:
        return list(BAIDU_SUPPORTED_VOICES)

    @classmethod
    def get_command(cls) -> click.Command:
        @click.command()
        @click.option(
            "--api-key",
            required=True,
            envvar="BAIDU_API_KEY",
            show_envvar=True,
            help="The Baidu API key.",
        )
        @click.option(
            "--secret-key",
            required=True,
            envvar="BAIDU_SECRET_KEY",
            show_envvar=True,
            help="The Baidu secret key.",
        )
        @click.option("--voice", help="The voice to use.")
        @click.option("--speed", default=5, help="The speed of speech.", type=click.INT)
        @click.option("--pitch", default=5, help="The pitch of speech.", type=click.INT)
        @click.option(
            "--volume", default=5, help="The volume of speech.", type=click.INT
        )
        @common_options(cls)
        def baidu(
            api_key: str,
            secret_key: str,
            voice: str,
            speed: int,
            pitch: int,
            volume: int,
            lang: str,
            text: str,
            output: str,
        ):
            speaker = BaiduSpeaker(api_key, secret_key, voice, speed, pitch, volume)
            speaker.say(text, output, lang=lang)

        return baidu
